@@ -5,6 +5,7 @@ geom_pointrange <- function(mapping = NULL, data = NULL,
                             ...,
                             fatten = 4,
                             na.rm = FALSE,
+                            orientation = NA,
                             show.legend = NA,
                             inherit.aes = TRUE) {
   layer(
@@ -18,6 +19,7 @@ geom_pointrange <- function(mapping = NULL, data = NULL,
     params = list(
       fatten = fatten,
       na.rm = na.rm,
+      orientation = orientation,
       ...
     )
   )
@@ -33,15 +35,25 @@ GeomPointrange <- ggproto("GeomPointrange", Geom,
 
   draw_key = draw_key_pointrange,
 
-  required_aes = c("x", "y", "ymin", "ymax"),
+  required_aes = c("x", "y", "ymin|xmin", "ymax|xmax"),
 
-  draw_panel = function(data, panel_params, coord, fatten = 4) {
-    if (is.null(data$y))
-      return(GeomLinerange$draw_panel(data, panel_params, coord))
+  setup_params = function(data, params) {
+    GeomLinerange$setup_params(data, params)
+  },
+
+  extra_params = c("na.rm", "orientation"),
+
+  setup_data = function(data, params) {
+    GeomLinerange$setup_data(data, params)
+  },
+
+  draw_panel = function(data, panel_params, coord, fatten = 4, flipped_aes = FALSE) {
+    if (is.null(data[[flipped_names(flipped_aes)$y]]))
+      return(GeomLinerange$draw_panel(data, panel_params, coord, flipped_aes = flipped_aes))
 
     ggname("geom_pointrange",
       gTree(children = gList(
-        GeomLinerange$draw_panel(data, panel_params, coord),
+        GeomLinerange$draw_panel(data, panel_params, coord, flipped_aes = flipped_aes),
         GeomPoint$draw_panel(transform(data, size = size * fatten), panel_params, coord)
       ))
     )
