@@ -6,7 +6,7 @@ test_that("can do frequency polygon with categorical x", {
   p <- ggplot(df, aes(x)) + geom_freqpoly(stat = "count")
   d <- layer_data(p)
 
-  expect_is(d$x, "integer")
-  expect_equal(d$x, 1:3)
+  expect_s3_class(d$x, "mapped_discrete")
+  expect_equal(d$x, new_mapped_discrete(1:3))
   expect_equal(d$y, 3:1)
 })
