@@ -42,8 +42,8 @@ test_that("modifying theme element properties with + operator works", {
 })
 
 test_that("adding theme object to ggplot object with + operator works", {
-
-  p <- qplot(1:3, 1:3)
+  ## test with complete theme
+  p <- qplot(1:3, 1:3) + theme_grey()
   p <- p + theme(axis.title = element_text(size = 20))
   expect_true(p$theme$axis.title$size == 20)
 
@@ -55,6 +55,36 @@ test_that("adding theme object to ggplot object with + operator works", {
   expect_true(tt$inherit.blank)
   tt$inherit.blank <- FALSE
   expect_identical(p$theme$text, tt)
+
+  ## test without complete theme
+  p <- qplot(1:3, 1:3)
+  p <- p + theme(axis.title = element_text(size = 20))
+  expect_true(p$theme$axis.title$size == 20)
+
+  # Should update specified properties, but not reset other properties
+  p <- p + theme(text = element_text(colour = 'red'))
+  expect_true(p$theme$text$colour == 'red')
+  expect_null(p$theme$text$family)
+  expect_null(p$theme$text$face)
+  expect_null(p$theme$text$size)
+  expect_null(p$theme$text$hjust)
+  expect_null(p$theme$text$vjust)
+  expect_null(p$theme$text$angle)
+  expect_null(p$theme$text$lineheight)
+  expect_null(p$theme$text$margin)
+  expect_null(p$theme$text$debug)
+
+  ## stepwise addition of partial themes is identical to one-step addition
+  p <- qplot(1:3, 1:3)
+  p1 <- p + theme_light() +
+    theme(axis.line.x = element_line(color = "blue")) +
+    theme(axis.ticks.x = element_line(color = "red"))
+
+  p2 <- p + theme_light() +
+    theme(axis.line.x = element_line(color = "blue"),
+          axis.ticks.x = element_line(color = "red"))
+
+  expect_identical(p1$theme, p2$theme)
 })
 
 test_that("replacing theme elements with %+replace% operator works", {
@@ -99,6 +129,49 @@ test_that("calculating theme element inheritance works", {
   # Check that a theme_blank in a parent node gets passed along to children
   t <- theme_grey() + theme(text = element_blank())
   expect_identical(calc_element('axis.title.x', t), element_blank())
+
+  # Check that inheritance from derived class works
+  element_dummyrect <- function(dummy) { # like element_rect but w/ dummy argument
+    structure(list(
+      fill = NULL, colour = NULL, dummy = dummy, size = NULL,
+      linetype = NULL, inherit.blank = FALSE
+    ), class = c("element_dummyrect", "element_rect", "element"))
+  }
+
+  e <- calc_element(
+    "panel.background",
+    theme(
+      rect = element_rect(fill = "white", colour = "black", size = 0.5, linetype = 1),
+      panel.background = element_dummyrect(dummy = 5),
+      complete = TRUE # need to prevent pulling in default theme
+    )
+  )
+
+  expect_identical(
+    e,
+    structure(list(
+      fill = "white", colour = "black", dummy = 5, size = 0.5, linetype = 1,
+      inherit.blank = TRUE # this is true because we're requesting a complete theme
+    ), class = c("element_dummyrect", "element_rect", "element"))
+  )
+
+  # Check that blank elements are skipped in inheritance tree if and only if elements
+  # don't inherit from blank.
+  t <- theme_gray() +
+    theme(
+      strip.text = element_blank(),
+      strip.text.x = element_text() # inherit.blank = FALSE is default
+    )
+  e1 <- calc_element("strip.text.x", t)
+  e2 <- calc_element("text", t)
+  e2$inherit.blank <- FALSE # b/c inherit.blank = TRUE for complete themes
+  expect_identical(e1, e2)
+
+  theme <- theme_gray() +
+    theme(strip.text = element_blank(), strip.text.x = element_text(inherit.blank = TRUE))
+  e1 <- ggplot2:::calc_element("strip.text.x", theme)
+  e2 <- ggplot2:::calc_element("strip.text", theme)
+  expect_identical(e1, e2)
 })
 
 test_that("complete and non-complete themes interact correctly with each other", {
@@ -173,6 +246,50 @@ test_that("theme(validate=FALSE) means do not validate_element", {
   expect_equal(red.before$theme$animint.width, 500)
 })
 
+test_that("theme validation happens at build stage", {
+  # adding a non-valid theme element to a theme is no problem
+  expect_silent(theme_gray() + theme(text = 0))
+
+  # the error occurs when we try to render the plot
+  p <- ggplot() + theme(text = 0)
+  expect_error(print(p), "must be an object of type `element_text`")
+
+  # without validation, the error occurs when the element is accessed
+  p <- ggplot() + theme(text = 0, validate = FALSE)
+  expect_error(print(p), "text should have class element_text")
+})
+
+test_that("element tree can be modified", {
+  # we cannot add a new theme element without modifying the element tree
+  p <- ggplot() + theme(blablabla = element_text(colour = "red"))
+  expect_error(print(p), "Theme element `blablabla` is not defined in the element hierarchy")
+
+  # things work once we add a new element to the element tree
+  register_theme_elements(
+    element_tree = list(blablabla = el_def("element_text", "text"))
+  )
+  expect_silent(print(p))
+
+  # inheritance and final calculation of novel element works
+  final_theme <- ggplot2:::plot_theme(p, theme_gray())
+  e1 <- calc_element("blablabla", final_theme)
+  e2 <- calc_element("text", final_theme)
+  expect_identical(e1$family, e2$family)
+  expect_identical(e1$face, e2$face)
+  expect_identical(e1$size, e2$size)
+  expect_identical(e1$lineheight, e2$lineheight)
+  expect_identical(e1$colour, "red") # not inherited from element_text
+
+  # existing elements can be overwritten
+  ed <- el_def("element_rect", "rect")
+  register_theme_elements(
+    element_tree = list(axis.title = ed)
+  )
+  expect_identical(get_element_tree()$axis.title, ed)
+
+  reset_theme_settings(reset_current = FALSE) # revert back to defaults
+})
+
 test_that("all elements in complete themes have inherit.blank=TRUE", {
   inherit_blanks <- function(theme) {
     all(vapply(theme, function(el) {
@@ -215,6 +332,12 @@ test_that("elements can be merged", {
   )
 })
 
+test_that("theme elements that don't inherit from element can be combined", {
+  expect_identical(combine_elements(1, NULL), 1)
+  expect_identical(combine_elements(NULL, 1), 1)
+  expect_identical(combine_elements(1, 0), 1)
+})
+
 test_that("complete plot themes shouldn't inherit from default", {
   default_theme <- theme_gray() + theme(axis.text.x = element_text(colour = "red"))
   base <- qplot(1, 1)
@@ -226,6 +349,44 @@ test_that("complete plot themes shouldn't inherit from default", {
   expect_null(ptheme$axis.text.x)
 })
 
+test_that("current theme can be updated with new elements", {
+  old <- theme_set(theme_grey())
+
+  b1 <- ggplot() + theme_grey()
+  b2 <- ggplot()
+
+  # works for root element
+  expect_identical(
+    calc_element("text", plot_theme(b1)),
+    calc_element("text", plot_theme(b2))
+  )
+
+  # works for derived element
+  expect_identical(
+    calc_element("axis.text.x", plot_theme(b1)),
+    calc_element("axis.text.x", plot_theme(b2))
+  )
+
+  # theme calculation for nonexisting element returns NULL
+  expect_identical(calc_element("abcde", plot_theme(b1)), NULL)
+
+  # element tree gets merged properly
+  register_theme_elements(
+    abcde = element_text(color = "blue", hjust = 0, vjust = 1),
+    element_tree = list(abcde = el_def("element_text", "text"))
+  )
+
+  e1 <- calc_element("abcde", plot_theme(b2))
+  e2 <- calc_element("text", plot_theme(b2))
+  e2$colour <- "blue"
+  e2$hjust <- 0
+  e2$vjust <- 1
+  expect_identical(e1, e2)
+
+  reset_theme_settings()
+  theme_set(old)
+})
+
 test_that("titleGrob() and margins() work correctly", {
   # ascenders and descenders
   g1 <- titleGrob("aaaa", 0, 0, 0.5, 0.5) # lower-case letters, no ascenders or descenders
@@ -263,6 +424,40 @@ test_that("titleGrob() and margins() work correctly", {
   expect_equal(width_cm(g10), width_cm(g1) + 2)
 })
 
+test_that("provided themes explicitly define all elements", {
+  elements <- names(.element_tree)
+
+  t <- theme_all_null()
+  expect_true(all(names(t) %in% elements))
+  expect_true(all(vapply(t, is.null, logical(1))))
+
+  t <- theme_grey()
+  expect_true(all(names(t) %in% elements))
+
+  t <- theme_bw()
+  expect_true(all(names(t) %in% elements))
+
+  t <- theme_linedraw()
+  expect_true(all(names(t) %in% elements))
+
+  t <- theme_light()
+  expect_true(all(names(t) %in% elements))
+
+  t <- theme_dark()
+  expect_true(all(names(t) %in% elements))
+
+  t <- theme_minimal()
+  expect_true(all(names(t) %in% elements))
+
+  t <- theme_classic()
+  expect_true(all(names(t) %in% elements))
+
+  t <- theme_void()
+  expect_true(all(names(t) %in% elements))
+
+  t <- theme_test()
+  expect_true(all(names(t) %in% elements))
+})
 
 # Visual tests ------------------------------------------------------------
 
@@ -394,3 +589,49 @@ test_that("rotated axis tick labels work", {
     theme(axis.text.x = element_text(angle = 50, hjust = 1))
   expect_doppelganger("rotated x axis tick labels", plot)
 })
+
+test_that("plot titles and caption can be aligned to entire plot", {
+  df <- data_frame(
+    x = 1:3,
+    y = 1:3,
+    z = letters[1:3]
+  )
+
+  plot <- ggplot(df, aes(x, y, color = z)) +
+    geom_point() + facet_wrap(~z) +
+    labs(
+      title = "Plot title aligned to entire plot",
+      subtitle = "Subtitle aligned to entire plot",
+      caption = "Caption aligned to panels"
+    ) +
+    theme(plot.title.position = "plot")
+  expect_doppelganger("titles aligned to entire plot", plot)
+
+  plot <- ggplot(df, aes(x, y, color = z)) +
+    geom_point() + facet_wrap(~z) +
+    labs(
+      title = "Plot title aligned to panels",
+      subtitle = "Subtitle aligned to panels",
+      caption = "Caption aligned to entire plot"
+    ) +
+    theme(plot.caption.position = "plot")
+  expect_doppelganger("caption aligned to entire plot", plot)
+
+})
+
+test_that("Strips can render custom elements", {
+  element_test <- function(...) {
+    el <- element_text(...)
+    class(el) <- c('element_test', 'element_text', 'element')
+    el
+  }
+  element_grob.element_test <- function(element, label = "", x = NULL, y = NULL, ...) {
+    rectGrob(width = unit(1, "cm"), height = unit(1, "cm"))
+  }
+  df <- data_frame(x = 1:3, y = 1:3, a = letters[1:3])
+  plot <- ggplot(df, aes(x, y)) +
+    geom_point() +
+    facet_wrap(~a) +
+    theme(strip.text = element_test())
+  expect_doppelganger("custom strip elements can render", plot)
+})
